#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Nov 14 08:38:21 2019

@author: dasanna
"""

import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
import matplotlib.gridspec as gs
import matplotlib.cbook as cbook
from skimage.filters import gaussian, median, threshold_local

from matplotlib.ticker import FormatStrFormatter


rc_para = {
    'lines.markersize': 3.0,
    'axes.labelsize': 16,
    'axes.titlesize': 20,
    'font.size': 18,
    # 'font.family': 'serif',
    'legend.fontsize': 15,
    'xtick.labelsize': 16,
    'ytick.labelsize': 16,
    'grid.linewidth': 0.8,
    'grid.alpha': 0.7,
    'axes.axisbelow': True,
    'text.usetex': True,
    'text.latex.preamble': [
        r'\usepackage[separate-uncertainty = true]{siunitx}',
        # r'\usepackage{helvet, times}',
        # i need upright \micro symbols, but you need...
        r'\sisetup{detect-all}',  # ...this to force siunitx to actually use your fonts
        # r'\usepackage[urw-garamond]{mathdesign}',  # set the normal font here
        # r'\usepackage{sansmath}',  # load up the sansmath so that math -> helvet
        # r'\sansmath'  # <- tricky! -- gotta actually tell tex to use!
    ]
}

sns.set_style('darkgrid', {'axes.linewidth': 0.5})
sns.set(style='ticks',rc=rc_para)



def get_figsize():
    return [6.4, 4]


def create_figure_for_data(data, label=None, fit=None):
    figure = plt.figure(figsize=get_figsize())
    nrow=3
    ncol=3
    grid = gs.GridSpec(nrow,ncol,hspace=0.1,bottom=0.2,left=0.15)

    axis_hist = figure.add_subplot(grid[1:, :])
    axis_box = figure.add_subplot(grid[0, :], sharex=axis_hist)

    axis_box = sns.boxplot(y=data, orient="h", ax=axis_box, sym='',color='red')

    axis_hist = sns.distplot(data, ax=axis_hist, norm_hist=True, label=label, kde=False, fit=fit,color='red')

    axis_box.get_xaxis().set_visible(False)

    return figure, axis_hist, axis_box


def texify_labels(labels, decimals):
    """
    Forces matplotlib to use LaTex to render axis labels
    :param labels: Labels to render
    :param decimals: Number of decimals used for representation
    :return: New labels
    """
    new_labels = [f"$\\num{{ {label:.{decimals}f} }}$" for label in labels]

    return new_labels


def plot_distance_angle_distribution(distance, angle, bin_width_distance, bin_width_angle, sigma=0.2):
    """
    Calculates and plots 2D histogram for head distance and alignment angle
    :param distance: All head distance measurements for various simulations of one parameter set
    :param angle: All alignment angles for various simulations of one parameter set
    :param bin_width_distance: Bin width of the histogram in distance direction
    :param bin_width_angle: Bin width of the histogram in angle direction
    :param rc_para: Further parameter to modify matplotlib behaviour
    :param sigma: Interaction length of the parasite-rbc interaction
    :param style: Seaborn plot parameter
    :return: figure with plotted histogram, numerical histogram, bin edges distance direction, bin edges angle direction
    """
    distance_stat = cbook.boxplot_stats(distance)[0]
    distance_mask = np.logical_and(distance_stat['whislo'] <= distance,
                                   distance_stat['whishi'] >= distance)

    angle_stat = cbook.boxplot_stats(angle)[0]
    angle_mask = np.logical_and(angle_stat['whislo'] <= angle,
                                angle_stat['whishi'] >= angle)

    total_mask = np.logical_and(distance_mask, angle_mask)

    # distance = distance[total_mask]
    # angle = angle[total_mask]

    bin_distance = np.arange(sigma, distance.max() + bin_width_distance, bin_width_distance)

    bin_angle = np.arange(0, 1.0 + bin_width_angle, bin_width_angle)


    

    figure, axis = plt.subplots(figsize=[8, 8])

    vmin = 0.15

    cmap = plt.cm.inferno
    cmap.set_under((1.0, 1.0, 1.0, 0.0))

    histogram, xedges, yedges = np.histogram2d(x=distance,
                                               y=angle,
                                               bins=[bin_distance, bin_angle],
                                               normed=True,
                                               )

    X, Y = np.meshgrid(xedges, yedges)

    histogram = gaussian(histogram, 0.5, mode='constant', cval=0.0)
    histogram *= bin_width_distance * bin_width_angle

    vmin *= bin_width_distance * bin_width_angle

    histogram[histogram <= vmin] = 0.0

    mesh = axis.pcolormesh(X, Y, histogram.T, cmap=cmap, vmin=vmin, zorder=3, edgecolor='white', linewidth=0.01)
    axis.set_xlim(xedges[0], xedges[-1])
    axis.set_ylim(yedges[0], yedges[-1])

    cb = plt.colorbar(mesh, ax=axis)

    cb.set_label('$P(d_\mathrm{apex}, \\theta/\pi)$', labelpad=10)

    effective_range = 1.12 * sigma
    x_criteria = effective_range + 0.008
    y_criteria = 0.8

    ylim = axis.get_ylim()

    axis.fill_between([0, x_criteria],
                      [y_criteria, y_criteria],
                      [ylim[1], ylim[1]],
                      color='palegreen',
                      alpha=0.3,
                      lw=0,
                      # edgecolor='red',
                      zorder=5)

    axis.plot([effective_range, effective_range], ylim, '--', color='black', alpha=0.7, zorder=1)

    # g.ax_marg_y.remove()

    axis.set_xlabel(r'$d \text{ in } \si{\micro\meter}$')
    axis.set_xlabel(r'$d_\mathrm{apex} / D_0$')
    axis.set_ylabel(r'$\theta/\pi$')


    distance_lim = distance.copy()[total_mask]
    angle_lim = angle.copy()[total_mask]

    xlim = [distance_lim.min() - 1.2 * bin_width_distance, distance_lim.max() + 8 * bin_width_distance]
    ylim = [angle_lim.min() - 4 * bin_width_angle, min(angle_lim.max() + 4 * bin_width_distance, 1.0)]

    axis.set_xlim(xlim)
    axis.set_ylim(ylim)

    D_0 = 6.5

    xticks = axis.get_xticks()
    xticks = [x/D_0 for x in xticks]

    axis.set_xticklabels(texify_labels(xticks, 2))
    axis.set_yticklabels(texify_labels(axis.get_yticks(), 2))
    cb.ax.set_yticklabels(texify_labels(cb.ax.get_yticks(), 3))

    return figure, histogram, xedges, yedges


distances = np.loadtxt('distances_d.dat')
angles = np.loadtxt('angles_d.dat')
bin_width_distance = 0.016
bin_width_angle = 0.021739130434782608
        

sigma = 0.2

g, histogram, xedges, yedges = plot_distance_angle_distribution(distances,
                                                                angles,
                                                                bin_width_distance,
                                                                bin_width_angle,
                                                                sigma=sigma)

g.tight_layout()
g.savefig('Fig4_a.pdf', transparent=True)

